@@ -1,22 +1,19 @@
-# $Id: dea.direct.R 85 2010-11-06 22:18:27Z Lars $
+# $Id: dea.direct.R 94 2010-11-21 23:50:53Z Lars $
 
 
 dea.direct <- function(X,Y, DIRECT, RTS="vrs", ORIENTATION="in", 
                   XREF=NULL, YREF=NULL, FRONT.IDX=NULL, SLACK=FALSE, 
                   TRANSPOSE=FALSE)  {
 
-   .xyref.missing <- FALSE
    if ( missing(XREF) || is.null(XREF) )  {
-      .xyref.missing <- TRUE
       XREF <- X
    }
    if ( missing(YREF) || is.null(YREF) )  {
-      .xyref.missing <- TRUE && .xyref.missing
       YREF <- Y
    }
    
    transpose <- FALSE
-   if ( !TRANSPOSE )  {
+   if ( TRANSPOSE )  {
       transpose <- TRUE
       X <- t(X)
       Y <- t(Y)
@@ -26,36 +23,34 @@ dea.direct <- function(X,Y, DIRECT, RTS="vrs", ORIENTATION="in",
          DIRECT <- t(DIRECT)
    }
 
-   m <- dim(X)[1]  # number of inputs
-   n <- dim(Y)[1]  # number of outputs
-   K <- dim(X)[2]  # number of units, firms, DMUs
-   Kr <- dim(XREF)[2] # number of units,firms in the reference technology
-
-
-
-ee <- dea(X,Y, RTS=RTS, ORIENTATION=ORIENTATION, XREF=XREF, YREF=YREF,
-         FRONT.IDX=FRONT.IDX, SLACK=SLACK, DUAL=FALSE, 
-         DIRECT=DIRECT, TRANSPOSE=TRUE)  
+   m <- dim(X)[2]  # number of inputs
+   n <- dim(Y)[2]  # number of outputs
+   K <- dim(X)[1]  # number of units, firms, DMUs
+   Kr <- dim(XREF)[1] # number of units,firms in the reference technology
 
+   ee <- dea(X,Y, RTS=RTS, ORIENTATION=ORIENTATION, XREF=XREF, YREF=YREF,
+             FRONT.IDX=FRONT.IDX, SLACK=SLACK, DUAL=FALSE, 
+             DIRECT=DIRECT, TRANSPOSE=FALSE)  
 
    mmd <- switch(ORIENTATION, "in"=m, "out"=n, "in-out"=m+n) 
-   ob <- matrix(ee$objval,nrow=mmd, ncol=K, byrow=T)
-   if ( class(DIRECT)=="matrix" && dim(DIRECT)[2] > 1 )  {
+   if ( is.null(ee$objval) )  ee$objval <- ee$eff
+   ob <- matrix(ee$objval,nrow=K, ncol=mmd)
+   if ( class(DIRECT)=="matrix" && dim(DIRECT)[1] > 1 )  {
        dir <- DIRECT
    } else {
-       dir <- matrix(DIRECT,nrow=mmd, ncol=K)
+       dir <- matrix(DIRECT,nrow=K, ncol=mmd, byrow=TRUE)
    }
    if ( ORIENTATION=="in" )  {
       e <- 1 - ob*dir/X
    } else if ( ORIENTATION=="out" )  {
       e <- 1 + ob*dir/Y
    } else if ( ORIENTATION=="in-out" )  {
-      e <- cbind(1 - ob[1:m,,drop=FALSE]*dir[1:m,,drop=FALSE]/X, 
-        1 + ob[(m+1):(m+n),,drop=FALSE]*dir[(m+1):(m+n),,drop=FALSE]/Y)
+      e <- cbind(1 - dir[,1:m,drop=FALSE]*ob[,1:m,drop=FALSE]/X, 
+        1 + dir[,(m+1):(m+n),drop=FALSE]*ob[,(m+1):(m+n),drop=FALSE]/Y)
    } else {
       warning("Illegal ORIENTATION for argument DIRECT") 
    }
-   if ( class(e)=="matrix" && ( dim(e)[1]==1 || dim(e)==1 ) )
+   if ( class(e)=="matrix" && dim(e)[2]==1 )
       e <- c(e) 
 
    if ( transpose )  {
@@ -64,7 +59,7 @@ ee <- dea(X,Y, RTS=RTS, ORIENTATION=ORIENTATION, XREF=XREF, YREF=YREF,
    } 
 
 
-   if ( !TRANSPOSE ) {
+   if ( TRANSPOSE ) {
       if ( class(e)=="matrix" )
          e <- t(e)
       ee$lambda <- t(ee$lambda)
